package org.kaoriha.jspbp;

import java.io.IOException;

import javax.servlet.ServletException;

import junit.framework.Assert;
import junit.framework.TestCase;

import org.apache.commons.lang.RandomStringUtils;
import org.kaoriha.jspbp.test.MockFilterChain;
import org.kaoriha.jspbp.test.MockFilterConfig;
import org.kaoriha.jspbp.test.MockRequest;
import org.kaoriha.jspbp.test.MockResponse;
import org.kaoriha.jspbp.test.TestCon;
import org.kaoriha.jspbp.test.Test;
import org.kaoriha.jspbp.test.TestList;
import org.kaoriha.jspbp.test.TestPer;
import org.kaoriha.jspbp.test.TestSC;

public class RequestTest extends TestCase {
	static final String behindPackageName = "org.kaoriha.jspbp.test";

	private JspbpFilter getFilter(String bpn) throws ServletException {
		JspbpFilter filter = new JspbpFilter();
		filter.init(new MockFilterConfig(bpn));
		return filter;
	}

	public void testSimpleGet() throws IOException, ServletException {
		MockRequest request = new MockRequest("/test.jsp", "GET");
		MockResponse response = new MockResponse();
		MockFilterChain chain = new MockFilterChain();

		Test.isCalled = false;
		new Request(request, response, chain, getFilter(behindPackageName)).doFilter();
		Assert.assertTrue(Test.isCalled);
		Assert.assertTrue(chain.isCalled);
		Object b = request.attributeMap.get("behind");
		Assert.assertTrue(b instanceof Test);

		Test.isCalled = false;
		new Request(request, response, chain, getFilter("org.kaoriha.jspbp.bad"))
			.doFilter();
		Assert.assertTrue(!Test.isCalled);

		request = new MockRequest("/testNoInterface.jsp", "GET");
		new Request(request, response, chain, getFilter(behindPackageName)).doFilter();
		Assert.assertEquals(405, response.error);
	}

	public void testSimplePost() throws IOException, ServletException {
		MockRequest request = new MockRequest("/test.jsp", "POST");
		MockResponse response = new MockResponse();
		MockFilterChain chain = new MockFilterChain();

		Test.isPostCalled = false;
		new Request(request, response, chain, getFilter("org.kaoriha.jspbp.test"))
				.doFilter();
		Assert.assertTrue(Test.isPostCalled);

		request = new MockRequest("/testNoInterface.jsp", "POST");
		new Request(request, response, chain, getFilter(behindPackageName)).doFilter();
		Assert.assertEquals(405, response.error);
	}

	public void testCopyListParameter() throws IOException, ServletException {
		MockRequest request = new MockRequest("/testList.jsp", "GET");
		String[] sa = {"a", "b"};
		request.parameterMap.put("test", sa);
		MockResponse response = new MockResponse();
		MockFilterChain chain = new MockFilterChain();

		TestList.testList = null;
		new Request(request, response, chain, getFilter("org.kaoriha.jspbp.test"))
				.doFilter();
		Assert.assertNotNull(TestList.testList);
		Assert.assertTrue(TestList.testList.size() == 2);
		Assert.assertTrue(TestList.testList.get(0).equals("a"));
		Assert.assertTrue(TestList.testList.get(1).equals("b"));
	}

	public void testReceiveContinuation() throws IOException, ServletException {
		String continueId = RandomStringUtils.randomAlphanumeric(20);

		Object cobj = "test";
		MockRequest request = new MockRequest("/test.jsp", "GET");
		String[] v = { continueId };
		request.parameterMap.put("continuationId", v);
		request.session.attributeMap.put(continueId, cobj);
		MockResponse response = new MockResponse();
		MockFilterChain chain = new MockFilterChain();

		Test.isCalled = false;
		Test.continuation = null;
		new Request(request, response, chain, getFilter(behindPackageName)).doFilter();
		Assert.assertTrue(Test.isCalled);
		Assert.assertTrue(chain.isCalled);
		Assert.assertTrue(cobj == Test.continuation);
	}

	public void testPutContinuation() throws IOException, ServletException {
		MockRequest request = new MockRequest("/testCon.jsp", "POST");
		MockResponse response = new MockResponse();
		MockFilterChain chain = new MockFilterChain();

		Object c = "test";
		TestCon.location = "test.jsp";
		TestCon.continuation = c;
		new Request(request, response, chain, getFilter("org.kaoriha.jspbp.test"))
				.doFilter();
		Assert.assertTrue(!response.redirectLocation.endsWith("testCon.jsp"));
		String cid = response.redirectLocation.split("=")[1];
		Assert.assertTrue(request.session.attributeMap.get(cid) == c);
	}

	public void testSendPersistentRedirect() throws IOException,
			ServletException {
		MockRequest request = new MockRequest("/testPer.jsp", "GET");
		MockResponse response = new MockResponse();
		MockFilterChain chain = new MockFilterChain();

		new Request(request, response, chain, getFilter(behindPackageName)).doFilter();

		Assert.assertTrue(!chain.isCalled);
		Assert.assertTrue(!response.redirectLocation.endsWith("testPer.jsp"));
		String[] sa = response.redirectLocation.split("[?=]");
		String qn = sa[1];
		Assert.assertEquals("persistentId", qn);
		String pid = sa[2];
		Assert.assertTrue(request.session.attributeMap.get(pid) == null);
	}

	public void testSendPersistentRedirect2() throws IOException,
			ServletException {
		String continueId = RandomStringUtils.randomAlphanumeric(20);
		Object cobj = "test";
		String[] v = { continueId };

		MockRequest request = new MockRequest("/testPer.jsp", "GET");
		request.parameterMap.put("continuationId", v);
		request.session.attributeMap.put(continueId, cobj);

		MockResponse response = new MockResponse();
		MockFilterChain chain = new MockFilterChain();

		new Request(request, response, chain, getFilter(behindPackageName)).doFilter();

		Assert.assertTrue(!chain.isCalled);
		Assert.assertTrue(!response.redirectLocation.endsWith("testPer.jsp"));
		String[] sa = response.redirectLocation.split("[?=&]");
		Assert.assertTrue("persistentId".equals(sa[1]) || "persistentId".equals(sa[3]));
		Assert.assertTrue(request.session.attributeMap.get(sa[2]) == null
				|| request.session.attributeMap.get(sa[4]) == null);
	}

	public void testPutPersistent() throws IOException, ServletException {
		String pid = RandomStringUtils.randomAlphanumeric(20);
		MockRequest request = new MockRequest("/testPer.jsp", "POST");
		String[] pa = { pid };
		request.parameterMap.put("persistentId", pa);
		MockResponse response = new MockResponse();
		MockFilterChain chain = new MockFilterChain();

		new Request(request, response, chain, getFilter("org.kaoriha.jspbp.test"))
				.doFilter();

		Assert.assertTrue(chain.isCalled);
		Assert.assertTrue(request.session.attributeMap.get(pid) != null);
	}

	public void testReceivePersistent() throws IOException, ServletException {
		String pid = RandomStringUtils.randomAlphanumeric(20);
		TestPer bb = new TestPer();
		MockRequest request = new MockRequest("/testPer.jsp", "POST");
		String[] pa = { pid };
		request.parameterMap.put("persistentId", pa);
		request.session.attributeMap.put(pid, bb);
		MockResponse response = new MockResponse();
		MockFilterChain chain = new MockFilterChain();

		new Request(request, response, chain, getFilter("org.kaoriha.jspbp.test"))
				.doFilter();

		Assert.assertTrue(request.session.attributeMap.get(pid) == bb);
		Assert.assertTrue(bb.isCalled);
		Assert.assertTrue(bb.persistentId.equals(pid));
	}


	public void testReceiveServletContext() throws IOException, ServletException {
		MockRequest request = new MockRequest("/testSC.jsp", "GET");
		MockResponse response = new MockResponse();
		MockFilterChain chain = new MockFilterChain();

		JspbpFilter filter = new JspbpFilter();
		MockFilterConfig fc = new MockFilterConfig(behindPackageName);
		filter.init(fc);

		new Request(request, response, chain, filter).doFilter();
		Assert.assertTrue(filter.getServletContext() == TestSC.getServletContextS());
	}
}
